/**********************************************************************
 * File Name: RadsimCanvas.java
 *
 * Copyright (C) 2000, US Air Force Research Laboratory,
 * Rome Research Site, Rome NY 13424, USA.
 *
 * Use of this software implies acceptance of the terms described in
 * accompanying license.txt file.
 *
 *
 * Authors: James Lawton, Jennifer Dzek, John Crowter,
 *          Bob Paragi, and Joe Beasock
 *
 * Creation Date: July 1999
 *
 * Description: The drawing canvas for the graphics for the
 * RadSim simulation.
 *
 * Modification History:
 *
 *  - June 00, Bryan Horling, UMass (BCH): Changes to make drawing more 
 *  efficient by only redrawing when needed.
 *
 **********************************************************************/

import java.util.*;
import java.awt.*;
import java.awt.event.*;

public class RadsimCanvas extends Canvas
{
    public SimEnvironment env;
    public PopupMenu infoMenu = null;

    public RadsimCanvas () {
 	offscreen = null;
    }

    protected Image offscreen, backscreen; /** We use page flipping to avoid flickering */
    protected Dimension offscreensize;
    protected java.awt.Graphics offgraphics,
	backgraphics; // BCH


    public void initComponents() {
	infoMenu = new PopupMenu("Object Info");
	add(infoMenu);

	addMouseListener(new MouseAdapter() {
		public void mouseClicked(MouseEvent e) {
		    handleMouseClick(e);
		}
	    });
    }

    public void update(Graphics g) {
        Dimension d = getSize();

        if ((offscreen == null) || (d.width != 	offscreensize.width) ||
            (d.height != offscreensize.height)) {
            offscreen = createImage(d.width, d.height);
            offscreensize = d;
            offgraphics = offscreen.getGraphics();
            offgraphics.setFont(getFont());

            backscreen = createImage(d.width, d.height);
            backgraphics = backscreen.getGraphics();
            backgraphics.setFont(getFont());

            env.requestRedraw();
            env.requestFullRedraw();
        }

	// Only redraw when needed (BCH)
        if (env.needsRedraw()) {
            draw(g, env.needsFullRedraw());
        } else {
            g.drawImage(offscreen,0,0,null);
        }
    }

    public void draw(Graphics g, boolean all) {
        Dimension d = getSize();

        int screen_height = d.height; // - 50;
        int screen_width = d.width;
        SimObject obj;

        //offgraphics.setColor(Color.black);
        //offgraphics.drawString("SimTime: " + env.clock.simtime, 10, 20);

        if (all) {
            backgraphics.setColor(Color.white);
            backgraphics.fillRect(0, 0, d.width, d.height);

            //System.out.println("-- Painting Screen.  Width = " + screen_width
            // + " Height = " + screen_height + " --");
            for (ListIterator objs = env.sensors.listIterator(); objs.hasNext(); ) {
                obj = (SimObject) objs.next();
                if (env.Debug > 1) System.out.println("Drawing object " + obj.name);
                obj.drawSelf(backgraphics, screen_width, screen_height);
                try {Thread.yield();} catch (Exception e){};
            }

            // Draw the scale bar (BCH)
            int scale = 20,
                div = 4,
                x = 15,
                y = screen_height - 10,
                width = env.scale_x_coord(scale, screen_width);
            String zstr = String.valueOf(0),
                sstr = String.valueOf(scale) + " feet";
            FontMetrics fm = backgraphics.getFontMetrics();
            backgraphics.setColor(Color.black);
            backgraphics.drawString(zstr, x - fm.stringWidth(zstr) - 2, y + fm.getAscent() / 2);
            backgraphics.drawLine(x, y, x+((width / div) * div), y);
            backgraphics.drawString(sstr, x+width+2, y + fm.getAscent() / 2);
            for (int i = 0; i <= div; i++) {
                backgraphics.drawLine(x, y-3, x, y+3);
                x += width / div;
            }
        }
        
        offgraphics.drawImage(backscreen,0,0,null);

        for (ListIterator objs = env.mobile_objects.listIterator(); objs.hasNext(); ) {
            obj = (SimObject) objs.next();
            if (env.Debug > 1) System.out.println("Drawing object " + obj.name);
            obj.drawSelf(offgraphics, screen_width, screen_height);
            try {Thread.yield();} catch (Exception e){};
        }
        g.drawImage(offscreen,0,0,null);

        env.handleRedraw();
    }

    public void handleMouseClick(MouseEvent e) {
  	Dimension d = getSize();
	double screen_height = (double) d.height;
  	double screen_width = (double) d.width;
	env.find_object_at((double) e.getX(), (double) e.getY(), screen_height, screen_width);
    }

}
